'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DB = exports.OPEN_READONLY = exports.OPEN_READWRITE = exports.OPEN_CREATE = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      Copyright 2016 Mozilla
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      Licensed under the Apache License, Version 2.0 (the "License"); you may not use
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      this file except in compliance with the License. You may obtain a copy of the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      License at http://www.apache.org/licenses/LICENSE-2.0
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      Unless required by applicable law or agreed to in writing, software distributed
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      CONDITIONS OF ANY KIND, either express or implied. See the License for the
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      specific language governing permissions and limitations under the License.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

/**
 * When life gives you callbacks, write your own promise wrapper.
 *
 * @module SQLite
 */

exports.verbose = verbose;

var _sqlite = require('sqlite3');

var _sqlite2 = _interopRequireDefault(_sqlite);

var _thenifyAll = require('thenify-all');

var _thenifyAll2 = _interopRequireDefault(_thenifyAll);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Promise = global.Promise;
var debug = false;

function verbose() {
  _sqlite2.default.verbose();
}

var OPEN_CREATE = exports.OPEN_CREATE = _sqlite2.default.OPEN_CREATE;
var OPEN_READWRITE = exports.OPEN_READWRITE = _sqlite2.default.OPEN_READWRITE;
var OPEN_READONLY = exports.OPEN_READONLY = _sqlite2.default.OPEN_READONLY;

/**
 * Turn methods on `source` that return callbacks into methods on `dest` that are bound to
 * `source` as if invoked as method calls.
 *
 * @param source an object with methods named by `methods`.
 * @param dest an object upon which attributes will be set.
 * @param methods an array of method names.
 */
function thenifyMethods(source, dest, methods) {
  var wrapped = (0, _thenifyAll2.default)(source, {}, methods);
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = methods[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var method = _step.value;

      dest[method] = wrapped[method].bind(source);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

var defaultLogger = {
  error: function error() {},
  info: function info() {}
};

var DB = exports.DB = function () {
  function DB(db, logger) {
    _classCallCheck(this, DB);

    this.db = db;
    this.logger = logger || defaultLogger;
    thenifyMethods(db, this, ['close', 'get', 'all', 'exec']);
  }

  _createClass(DB, [{
    key: 'prepare',
    value: function prepare(sql, params) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        var statement = _this.db.prepare(sql, params, function (err) {
          if (err) {
            reject(err);
          } else {
            resolve(statement);
          }
        });
      });
    }

    /**
     * Invokes the provided `onRow` callback with each result row.
     * Returns a promise that resolves to the number of returned rows.
     *
     * @param sql the query to run.
     * @param params any parameters to bind.
     * @param onRow a function of one argument, `(row)`.
     * @returns {Promise<int>} the number of returned rows.
     */

  }, {
    key: 'each',
    value: function each(sql, params, onRow) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var done = false;

        var rowCallback = function rowCallback(err, row) {
          if (done) {
            return;
          }

          if (err) {
            _this2.logger.error('SQL error in row function: ' + err + ' in ' + sql + '.');
            done = true;
            reject(err);
            return;
          }

          onRow(row);
        };

        var completionCallback = function completionCallback(err, count) {
          if (err) {
            _this2.logger.error('SQL error in completion function: ' + err + ' in ' + sql + '.');
            reject(err);
            return;
          }

          resolve(count);
        };

        _this2.db.each(sql, params, rowCallback, completionCallback);
      });
    }
  }, {
    key: 'run',
    value: function run(sql, params) {
      var _this3 = this;

      var logger = this.logger;
      return new Promise(function (resolve, reject) {
        _this3.db.run(sql, params, function (err) {
          if (debug) {
            logger.info('Running: ' + sql + ', ' + JSON.stringify(params));
          }

          if (err) {
            logger.info('SQL error: ' + err + ' in ' + sql + '.');
            reject(err);
          } else {
            resolve({ lastID: this.lastID, changes: this.changes });
          }
        });
      });
    }

    /**
     * Begins a transaction and runs `f`. If `f` returns a promise that rejects,
     * rolls back the transaction and passes through the rejection. If `f` resolves
     * to a value, commits the transaction and passes through the resolved value.
     *
     * @param f the function to call within a transaction.
     * @returns {*} the result or rejection of `f`.
     */

  }, {
    key: 'inTransaction',
    value: function inTransaction(f) {
      var _this4 = this;

      return this.run('BEGIN TRANSACTION').then(f).then(function (result) {
        return _this4.run('COMMIT').then(function () {
          return Promise.resolve(result);
        });
      }).catch(function (err) {
        return _this4.run('ROLLBACK').then(function () {
          return Promise.reject(err);
        });
      });
    }
  }, {
    key: 'close',
    value: function close() {
      var _this5 = this;

      return new Promise(function (resolve, reject) {
        _this5.db.close(function (err) {
          if (err) {
            reject(err);
          } else {
            resolve();
          }
        });
      });
    }

    /**
     *
     * @param filename the file path to open
     * @param mode any mode flags to pass to `sqlite3.Database`.
     *             Defaults to OPEN_CREATE | OPEN_READWRITE.  May be
     *             OPEN_READONLY, but be aware of https://www.sqlite.org/wal.html#readonly.
     * @param logger an object with 'info' and 'error' methods, each of which takes a string argument.
     *               Defaults to a placeholder that logs nothing.
     * @returns Promise that resolves to the opened database.
     */

  }], [{
    key: 'open',
    value: function open(filename) {
      var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref$mode = _ref.mode;
      var mode = _ref$mode === undefined ? OPEN_CREATE | OPEN_READWRITE : _ref$mode;
      var _ref$logger = _ref.logger;
      var logger = _ref$logger === undefined ? defaultLogger : _ref$logger;

      return new Promise(function (resolve, reject) {
        var sq = new _sqlite2.default.Database(filename, mode, function (err) {
          if (err) {
            reject(err);
          } else {
            resolve(new DB(sq, logger));
          }
        });
      });
    }
  }]);

  return DB;
}();